This is the mail archive of the gdb-patches@sources.redhat.com mailing list for the GDB project.


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Other format: [Raw text]

[PATCH] Add support for debugging 32-bit code on GNU/Linux x86-64


As promised.  There is now some duplication of information between
-tdep and -nat with respect to the register mappings.  It's my
intention to keep that duplication, and add a sanity check to -nat to
check whether things match up.  That way, if the offsets ever change,
we'll notice it when running a native GDB.

Committed.

Mark


Index: ChangeLog
from  Mark Kettenis  <kettenis@gnu.org>
 
	* i386-linux-tdep.h (I386_LINUX_NUM_REGS): New define.
	* x86-64-linux-nat.c: Include "i386-linux-tdep.h" and "amd64.h".
	Change "register array" to "register cache" in comments.
	(x86_64_linux_gregset64_reg_offset): New variable.
	(GETREGS_SUPPLIES): Remove macro.
	(supply_gregset): Call amd64_supply_native_gregset instead of
	x86_64_linux_supply_gregset.
	(fill_gregset): Rename `regno' to `regnum'.  Call
	amd64_collect_native_gregset instead of x86_64_linux_fill_gregset.
	(store_regs): Rename `regno' to `regnum'.
	(store_fpregs): Rename `regno' to `regnum'.
	(fetch_inferior_registers): Rename `regno' to `regnum'.  Use
	amd64_native_gregset_supplies_p instead of GREGSET_SUPPLIES.
	Reorganize function a bit.
	(store_inferior_registers): Rename `regno' to `regnum'.  Use
	amd64_native_gregset_supplies_p instead of GREGSET_SUPPLIES.
	Reorganize function a bit.
	(_initialize_x86_64_linux_nat): New function.
	* config/i386/x86-64linux.mh.

 
Index: i386-linux-tdep.h
===================================================================
RCS file: /cvs/src/src/gdb/i386-linux-tdep.h,v
retrieving revision 1.2
diff -u -p -r1.2 i386-linux-tdep.h
--- i386-linux-tdep.h 18 Sep 2002 22:00:22 -0000 1.2
+++ i386-linux-tdep.h 29 Aug 2003 16:12:19 -0000
@@ -1,6 +1,6 @@
 /* Target-dependent code for GNU/Linux x86.
 
-   Copyright 2002 Free Software Foundation, Inc.
+   Copyright 2002, 2003 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -32,5 +32,8 @@
    pseudo-register contains a value >= 0 it is interpreted as the
    system call number that the kernel is supposed to restart.  */
 #define I386_LINUX_ORIG_EAX_REGNUM I386_SSE_NUM_REGS
+
+/* Total number of registers for GNU/Linux.  */
+#define I386_LINUX_NUM_REGS (I386_LINUX_ORIG_EAX_REGNUM + 1)
 
 #endif /* i386-linux-tdep.h */
Index: config/i386/x86-64linux.mh
===================================================================
RCS file: /cvs/src/src/gdb/config/i386/x86-64linux.mh,v
retrieving revision 1.10
diff -u -p -r1.10 x86-64linux.mh
--- config/i386/x86-64linux.mh 15 Jul 2003 11:33:59 -0000 1.10
+++ config/i386/x86-64linux.mh 29 Aug 2003 16:12:19 -0000
@@ -4,7 +4,7 @@ XM_FILE= xm-i386.h
 
 NAT_FILE= nm-x86-64linux.h
 NATDEPFILES= infptrace.o inftarg.o fork-child.o \
-	i386-nat.o x86-64-linux-nat.o \
+	i386-nat.o amd64-nat.o x86-64-linux-nat.o \
 	linux-nat.o \
 	proc-service.o thread-db.o lin-lwp.o linux-proc.o gcore.o 
 


Index Nav: [Date Index] [Subject Index] [Author Index] [Thread Index]
Message Nav: [Date Prev] [Date Next] [Thread Prev] [Thread Next]
Index: x86-64-linux-nat.c
===================================================================
RCS file: /cvs/src/src/gdb/x86-64-linux-nat.c,v
retrieving revision 1.27
diff -u -p -r1.27 x86-64-linux-nat.c
--- x86-64-linux-nat.c 23 Aug 2003 13:58:18 -0000 1.27
+++ x86-64-linux-nat.c 29 Aug 2003 16:12:19 -0000
@@ -50,13 +50,56 @@
 
 #include "x86-64-tdep.h"
 #include "x86-64-linux-tdep.h"
+#include "i386-linux-tdep.h"
+#include "amd64-nat.h"
+
+/* Mapping between the general-purpose registers in GNU/Linux x86-64
+   `struct user' format and GDB's register cache layout.  */
+
+static int x86_64_linux_gregset64_reg_offset[] =
+{
+  RAX * 8, RBX * 8,		/* %rax, %rbx */
+  RCX * 8, RDX * 8,		/* %rcx, %rdx */
+  RSI * 8, RDI * 8,		/* %rsi, %rdi */
+  RBP * 8, RSP * 8,		/* %rbp, %rsp */
+  R8 * 8, R9 * 8,		/* %r8 ... */
+  R10 * 8, R11 * 8,
+  R12 * 8, R13 * 8,
+  R14 * 8, R15 * 8,		/* ... %r15 */
+  RIP * 8, EFLAGS * 8,		/* %rip, %eflags */
+  DS * 8, ES * 8,		/* %ds, %es */
+  FS * 8, GS * 8		/* %fs, %gs */
+};
+
+
+/* Mapping between the general-purpose registers in GNU/Linux x86-64
+   `struct user' format and GDB's register cache layout for GNU/Linux
+   i386.
+
+   Note that most GNU/Linux x86-64 registers are 64-bit, while the
+   GNU/Linux i386 registers are all 32-bit, but since we're
+   little-endian we get away with that.  */
+
+/* From <sys/reg.h> on GNU/Linux i386.  */
+static int x86_64_linux_gregset32_reg_offset[] =
+{
+  10 * 8, 11 * 8,		/* %eax, %ecx */
+  12 * 8, 13 * 8,		/* %edx, %ebx */
+  19 * 8, 4 * 8,		/* %esp, %ebp */
+  13 * 8, 14 * 8,		/* %esi, %edi */
+  16 * 8, 18 * 8,		/* %eip, %eflags */
+  17 * 8, 20 * 8,		/* %cs, %ss */
+  23 * 8, 24 * 8,		/* %ds, %es */
+  25 * 4, 26 * 4,		/* %fs, %gs */
+  -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1,
+  -1, -1, -1, -1, -1, -1, -1, -1, -1,
+  15 * 8			/* "orig_eax" */
+};
 
 /* Which ptrace request retrieves which registers?
    These apply to the corresponding SET requests as well.  */
 
-#define GETREGS_SUPPLIES(regno) \
-  (0 <= (regno) && (regno) < X86_64_NUM_GREGS)
-
 #define GETFPREGS_SUPPLIES(regno) \
   (FP0_REGNUM <= (regno) && (regno) <= MXCSR_REGNUM)
 
@@ -64,27 +107,27 @@
 /* Transfering the general-purpose registers between GDB, inferiors
    and core files.  */
 
 /* Fill GDB's register array with the general-purpose register values
    in *GREGSETP.  */
 
 void
 supply_gregset (elf_gregset_t *gregsetp)
 {
-  x86_64_linux_supply_gregset ((char *) gregsetp);
+  amd64_supply_native_gregset (current_regcache, gregsetp, -1);
 }
 
 /* Fill register REGNO (if it is a general-purpose register) in
    *GREGSETPS with the value in GDB's register array.  If REGNO is -1,
    do this for all registers.  */
 
 void
-fill_gregset (elf_gregset_t *gregsetp, int regno)
+fill_gregset (elf_gregset_t *gregsetp, int regnum)
 {
-  x86_64_linux_fill_gregset ((char *) gregsetp, regno);
+  amd64_collect_native_gregset (current_regcache, gregsetp, regnum);
 }
 
 /* Fetch all general-purpose registers from process/thread TID and
-   store their values in GDB's register array.  */
+   store their values in GDB's register cache.  */
 
 static void
 fetch_regs (int tid)
@@ -97,18 +140,18 @@ fetch_regs (int tid)
   supply_gregset (&regs);
 }
 
 /* Store all valid general-purpose registers in GDB's register array
    into the process/thread specified by TID.  */
 
 static void
-store_regs (int tid, int regno)
+store_regs (int tid, int regnum)
 {
   elf_gregset_t regs;
 
   if (ptrace (PTRACE_GETREGS, tid, 0, (long) &regs) < 0)
     perror_with_name ("Couldn't get registers");
 
-  fill_gregset (&regs, regno);
+  fill_gregset (&regs, regnum);
 
   if (ptrace (PTRACE_SETREGS, tid, 0, (long) &regs) < 0)
     perror_with_name ("Couldn't write registers");
@@ -117,7 +160,7 @@ store_regs (int tid, int regno)
 
 /* Transfering floating-point registers between GDB, inferiors and cores.  */
 
 /* Fill GDB's register array with the floating-point and SSE register
    values in *FPREGSETP.  */
 
 void
@@ -127,7 +170,7 @@ supply_fpregset (elf_fpregset_t *fpregse
 }
 
 /* Fill register REGNUM (if it is a floating-point or SSE register) in
    *FPREGSETP with the value in GDB's register array.  If REGNUM is
    -1, do this for all registers.  */
 
 void
@@ -137,7 +180,7 @@ fill_fpregset (elf_fpregset_t *fpregsetp
 }
 
 /* Fetch all floating-point registers from process/thread TID and store
-   thier values in GDB's register array.  */
+   thier values in GDB's register cache.  */
 
 static void
 fetch_fpregs (int tid)
@@ -150,18 +193,18 @@ fetch_fpregs (int tid)
   supply_fpregset (&fpregs);
 }
 
 /* Store all valid floating-point registers in GDB's register array
    into the process/thread specified by TID.  */
 
 static void
-store_fpregs (int tid, int regno)
+store_fpregs (int tid, int regnum)
 {
   elf_fpregset_t fpregs;
 
   if (ptrace (PTRACE_GETFPREGS, tid, 0, (long) &fpregs) < 0)
     perror_with_name ("Couldn't get floating point status");
 
-  fill_fpregset (&fpregs, regno);
+  fill_fpregset (&fpregs, regnum);
 
   if (ptrace (PTRACE_SETFPREGS, tid, 0, (long) &fpregs) < 0)
     perror_with_name ("Couldn't write floating point status");
@@ -170,12 +213,12 @@ store_fpregs (int tid, int regno)
 
 /* Transferring arbitrary registers between GDB and inferior.  */
 
 /* Fetch register REGNO from the child process.  If REGNO is -1, do
    this for all registers (including the floating point and SSE
    registers).  */
 
 void
-fetch_inferior_registers (int regno)
+fetch_inferior_registers (int regnum)
 {
   int tid;
 
@@ -184,35 +227,29 @@ fetch_inferior_registers (int regno)
   if (tid == 0)
     tid = PIDGET (inferior_ptid); /* Not a threaded program.  */
 
-  if (regno == -1)
-    {
-      fetch_regs (tid);
-      fetch_fpregs (tid);
-      return;
-    }
-
-  if (GETREGS_SUPPLIES (regno))
+  if (regnum == -1 || amd64_native_gregset_supplies_p (regnum))
     {
       fetch_regs (tid);
-      return;
+      if (regnum != -1)
+	return;
     }
 
-  if (GETFPREGS_SUPPLIES (regno))
+  if (regnum == -1 || GETFPREGS_SUPPLIES (regnum))
     {
       fetch_fpregs (tid);
       return;
     }
 
   internal_error (__FILE__, __LINE__,
-		  "Got request for bad register number %d.", regno);
+		  "Got request for bad register number %d.", regnum);
 }
 
-/* Store register REGNO back into the child process.  If REGNO is -1,
-   do this for all registers (including the floating-point and SSE
+/* Store register REGNUM back into the child process.  If REGNUM is
+   -1, do this for all registers (including the floating-point and SSE
    registers).  */
 
 void
-store_inferior_registers (int regno)
+store_inferior_registers (int regnum)
 {
   int tid;
 
@@ -221,27 +258,21 @@ store_inferior_registers (int regno)
   if (tid == 0)
     tid = PIDGET (inferior_ptid); /* Not a threaded program.  */
 
-  if (regno == -1)
-    {
-      store_regs (tid, regno);
-      store_fpregs (tid, regno);
-      return;
-    }
-
-  if (GETREGS_SUPPLIES (regno))
+  if (regnum == -1 || amd64_native_gregset_supplies_p (regnum))
     {
-      store_regs (tid, regno);
-      return;
+      store_regs (tid, regnum);
+      if (regnum != -1)
+	return;
     }
 
-  if (GETFPREGS_SUPPLIES (regno))
+  if (regnum == -1 || GETFPREGS_SUPPLIES (regnum))
     {
-      store_fpregs (tid, regno);
+      store_fpregs (tid, regnum);
       return;
     }
 
   internal_error (__FILE__, __LINE__,
-		  "Got request to store bad register number %d.", regno);
+		  "Got request to store bad register number %d.", regnum);
 }
 
 
@@ -353,4 +384,21 @@ child_post_startup_inferior (ptid_t ptid
 {
   i386_cleanup_dregs ();
   linux_child_post_startup_inferior (ptid);
+}
+
+
+/* Provide a prototype to silence -Wmissing-prototypes.  */
+void _initialize_x86_64_linux_nat (void);
+
+void
+_initialize_x86_64_linux_nat (void)
+{
+  amd64_native_gregset32_reg_offset = x86_64_linux_gregset32_reg_offset;
+  amd64_native_gregset32_num_regs = I386_LINUX_NUM_REGS;
+  amd64_native_gregset64_reg_offset = x86_64_linux_gregset64_reg_offset;
+
+  gdb_assert (ARRAY_SIZE (x86_64_linux_gregset32_reg_offset)
+	      == amd64_native_gregset32_num_regs);
+  gdb_assert (ARRAY_SIZE (x86_64_linux_gregset64_reg_offset)
+	      == amd64_native_gregset64_num_regs);
 }
@@ -64,27 +107,27 @@
    and core files.  */
 
-/* Fill GDB's register array with the general-purpose register values
+/* Fill GDB's register cache with the general-purpose register values
    in *GREGSETP.  */
 
@@ -64,27 +107,27 @@
 }
 
-/* Fill register REGNO (if it is a general-purpose register) in
-   *GREGSETPS with the value in GDB's register array.  If REGNO is -1,
+/* Fill register REGNUM (if it is a general-purpose register) in
+   *GREGSETP with the value in GDB's register cache.  If REGNUM is -1,
    do this for all registers.  */
 
@@ -97,18 +140,18 @@ fetch_regs (int tid)
 }
 
-/* Store all valid general-purpose registers in GDB's register array
+/* Store all valid general-purpose registers in GDB's register cache
    into the process/thread specified by TID.  */
 
@@ -117,7 +160,7 @@ store_regs (int tid, int regno)
 /* Transfering floating-point registers between GDB, inferiors and cores.  */
 
-/* Fill GDB's register array with the floating-point and SSE register
+/* Fill GDB's register cache with the floating-point and SSE register
    values in *FPREGSETP.  */
 
@@ -127,7 +170,7 @@ supply_fpregset (elf_fpregset_t *fpregse
 
 /* Fill register REGNUM (if it is a floating-point or SSE register) in
-   *FPREGSETP with the value in GDB's register array.  If REGNUM is
+   *FPREGSETP with the value in GDB's register cache.  If REGNUM is
    -1, do this for all registers.  */
 
@@ -150,18 +193,18 @@ fetch_fpregs (int tid)
 }
 
-/* Store all valid floating-point registers in GDB's register array
+/* Store all valid floating-point registers in GDB's register cache
    into the process/thread specified by TID.  */
 
@@ -170,12 +213,12 @@ store_fpregs (int tid, int regno)
 /* Transferring arbitrary registers between GDB and inferior.  */
 
-/* Fetch register REGNO from the child process.  If REGNO is -1, do
+/* Fetch register REGNUM from the child process.  If REGNUM is -1, do
    this for all registers (including the floating point and SSE
    registers).  */